--first function, LUP_Solve, solves a set of simultaneous equations given
--L,U,pi and b. These are returned, given your matrix of coefficients, A,
--by the second function.

--Ax=b is the input form, the second function solves the unique equation PA=LU
--where L and U are lower and upper triangular respectivly, and P is a matrix
--of pivots. (this is to prevent division by 0)
--Once L, U, P, A and b are present the following mathematics solves the
--equations

--Ax=b  PA=LU
--PAx = Pb by multiplying
--LUx = Pb by substitution
--if we let y = Ux
--then Ly = Pb
--as L is triangular, this equation is solved, for y, by forward substitution
--given y and U we can then solve y = Ux, as U is triangular,
--by back substitution
--this x is our solution, as shown below

--y = Ux
--Ly = LUx
--Ly = PAx as PA = LU
--Ly = Pb as Ax = b
--LUx = Pb as y = Ux
--PAx = Pb as PA = LU
--Ax = b

global integer dimension

function LUP_Solve(sequence L, sequence U, sequence P, sequence b)
	sequence y, x
	atom temp
	y = repeat(0, dimension)
	x = y
	temp = 0

	for i = 1 to dimension do --This for loop, solves Ly = Pb by forward
				  --substitution
		for j = 1 to i-1 do
			temp = temp + L[i][j]*y[j]
		end for

		y[i] = b[P[i]] - temp
		temp = 0

	end for

	for i = dimension to 1 by -1 do --This loop solves y = Ux by back
					--substitution
		for j = i+1 to dimension do
			temp = temp + U[i][j]*x[j]
		end for

		x[i] = (y[i] - temp)/U[i][i]
		temp = 0

	end for

return x
end function

function Solve_Simultaneous(sequence A, sequence b)
	sequence L, U, P
	object k_dash,p,temp
	P = repeat(0,dimension)
	L = repeat(repeat(0,dimension),dimension)
	U = L
	  temp = 0
	for i = 1 to dimension do
		P[i] = i
	end for
	for k = 1 to dimension-1 do --This loop decomposes A into L and U
				    --while dynamically maintaining P
		p = 0
		for i = k to dimension do
			if A[i][k] > p then
				p = A[i][k]
				k_dash = i
			end if

		end for
	
		temp = P[k]
		P[k] = P[k_dash]
		P[k_dash] = temp
		
		temp = A[k]
		A[k] = A[k_dash]
		A[k_dash] = temp

		for i = k+1 to dimension do
			A[i][k] = A[i][k]/A[k][k]
			for j = k+1 to dimension do
				A[i][j] = A[i][j] - A[i][k]*A[k][j]
			end for
		end for
	end for

	for i = 1 to dimension do --The first loop gave the significant entries
				  --of L and U, but not L and U themselves,
				  --so this loop puts them into two seperate
				  --matrices
		for j = 1 to dimension do
			if i > j then
				L[i][j] = A[i][j]
				U[i][j] = 0
			elsif i < j then
				U[i][j] = A[i][j]
				L[i][j] = 0
			elsif i = j then
				U[i][j] = A[i][j]
				L[i][j] = 1
			end if
		end for
	end for
return LUP_Solve(L,U,P,b)
end function


------------------------------------------------------
--------------------Test Code!!!----------------------
------------------------------------------------------
--sequence A --Matrix of coefficients
--sequence b --Column vector of results
--sequence x --Column vector of solutions
--A = {{1,1,1,1},{7,1,-3,4},{3,9,1,-10},{-2,-6,7,3}}
--b = {14,25,-13,21}
--if length(A[1]) = length(b) then
--dimension = length(A[1])
--x = Solve_Simultaneous(A,b)
--? x
--else
--puts(1, "Matrices are incorrect size!!!!")
--end if

